; Display PETSCII picture on hires screen. (with optional hires colours or flipped gfx)
; 4mat/2016 - Dasm format.

; You will need:
; --------------	

; 1) Your petscii picture page. (1k in size, like a normal text screen)
; 2) Colour ram page for the image. (1k in size, either just the single colour per byte or both colours stored in low/high nibble format like a hires picture)
; 3) Optional extra colour ram page for the hires extra colours.  (1k in size again)

; When running you can save memory from $2000-$4328 to get an Art Studio version of the displayed pic.  Most tools load Art Studio format for hires bitmaps.
; To save pic in Vice , enter the monitor (alt+M) and type  s "filename.prg" 0 2000 4328 to do that.

; Set the following options and then go to bottom of the source to put in filenames to load.

; Options:
; --------

; If you just want to display a standard petscii pic (without extended colours) set this to 1, otherwise leave it as 0.
STANDARDPETSCII = 0

; If you want to use two separate files for the color ram values set this to 1, otherwise leave it as 0 to use the color ram picture as nibbles. (low/high)
DUALCOLORRAM = 1

; Set border colour here. (0-15)
BORDERCOLOR = 0

; Set paper colour here. (0-15)  Will only be used when STANDARDPETSCII = 1
PAPERCOLOR = 0

; If you want to flip all chars vertically set this to 1.  (was done for someone else's picture but you might find it useful)
FLIPINY = 0

; Source starts:

	processor 6502

	org	$0801
	.byte	$0b,$08,$ef,$00,$9e,$32,$30,$36,$32,0,0,0,0
	org	$080e

	sei
		
	lda #$32
	sta $01
	ldx #$00
copygfx
	lda $d000,x
togfx
	sta $c800,x
	inx
	bne copygfx
	inc copygfx+$02
	inc togfx+$02
	lda copygfx+$02
	cmp #$d8
	bne copygfx
	lda #$35
	sta $01
	
	ldx #$00
fixcolorbase
	lda $2400,x
	.IF STANDARDPETSCII = 1
	asl
	asl
	asl
	asl
	and #$f0
	clc
	adc #PAPERCOLOR
	jmp fillcolorbase
	.ENDIF
	.IF DUALCOLORRAM = 1
	asl
	asl
	asl
	asl
	and #$f0
	sta temp
filldual	
	lda $2800,x
	and #$0f
	clc
	adc temp
	.ENDIF
fillcolorbase
	sta $c400,x
	inx
	cpx #$00
	bne fixcolorbase
	inc fixcolorbase+$02
	inc fillcolorbase+$02
	.IF DUALCOLORRAM = 1
	inc filldual+$02
	.ENDIF
	lda fixcolorbase+$02
	cmp #$28
	bne fixcolorbase
	
	.ENDIF
	
	ldx #$00

petsciiconvert
	ldx petsciicount
getpetsciichar
	lda $2000,x
	sta temp
	lsr
	lsr
	lsr
	lsr
	lsr
	clc
	adc #$c8
	sta readpetsciifont+$02
	lda temp
	and #$1f
	asl
	asl
	asl
	sta readpetsciifont+$01
	
	ldx #$00
	.IF FLIPINY = 0
	ldy #$00
	.ELSE
	ldy #$07
	.ENDIF
readpetsciifont 
	lda $f000,y
writepetsciifont
	sta $e000,x
	
	.IF FLIPINY = 0
	iny
	.ELSE
	dey
	.ENDIF
	
	inx
	cpx #$08
	bne readpetsciifont
	clc
	lda writepetsciifont+$01
	adc #$08
	sta writepetsciifont+$01
	cmp #$00
	bne nowriteadd
	inc writepetsciifont+$02
	lda writepetsciifont+$02
	cmp #$00
	beq stopdrawing
	
nowriteadd
	inc petsciicount
	lda petsciicount
	cmp #$00
	bne petsciiconvert
	inc getpetsciichar+$02
	jmp petsciiconvert
	
stopdrawing

	lda #$08
	sta $d016
	lda #63
	sta $dd02
	lda #148
	sta $dd00
	lda #196
	sta $0288
	lda #$18
	sta 53272
	lda #59
	sta 53265
		
	lda #BORDERCOLOR
	sta $d020
	
	; do conversion to art studio format
	
	ldx #$00
artloop
	lda $e000,x
artloop2
	sta $2000,x
	inx
	cpx #$00
	bne artloop
	inc artloop+$02
	inc artloop2+$02
	lda artloop+$02
	cmp #$00
	bne artloop
	ldx #$00
artcolor
	lda $c400,x
artcolor2
	sta $3f40,x
	inx
	cpx #$00
	bne artcolor
	inc artcolor+$02
	inc artcolor2+$02
	lda artcolor+$02
	cmp #$c8
	bne artcolor
	lda #BORDERCOLOR
	sta $4328

waithere
	
	jmp waithere


	
temp
	.byte $00
petsciicount
	.byte $00



; Set filenames here
	
	org $1ffe

	; Set name of your petscii picture.
	
	.incbin testcard_gfx.prg
	
	org $23fe
	
	; Set name of your petscii color ram.
	
	.incbin testcard_col_low.prg
	
	org $27fe
	
	.IF DUALCOLORRAM = 1
	
	; Set name of your extended color ram.  Only added if DUALCOLORRAM is enabled.
	
	.incbin testcard_col_high.prg
	
	.ENDIF

